//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

/*----------------------------------------------------------------------------

 recieve IR of NEC format, setup data to LED PWM(color chart) & send RS232 
 
 NEC format
 	reader on	9ms
	reader off	4.5ms
	data		32bit(custom 2byte/data 1byte/~data 1byte)
					  : '1' -> on 0.56ms off 1.68ms -> 2.24ms
				      : '0' -> on 0.56ms off 0.56ms -> 1.12ms
	stop		1bit

 function
	1. recieve IR of NEC format
	2. data code -> index of color chart
	3. PWM RGB LED at color chart index
	4. data code -> send RS232
	
----------------------------------------------------------------------------*/
#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules

#include "color.h"		// color chart header

//#include "psocdynamic.h"

#define	WAITING		1						// IR waiting condition
#define	READER		2						// IR Reader  condition 
#define	DATA		3						// IR data    condition 

#pragma interrupt_handler pwm8_1_isr		// pwm8 timer interrupt(139us)
#pragma interrupt_handler gpio_isr			// IR interrupt
#pragma interrupt_handler timer8_1_isr		// IR capture interrupt

volatile unsigned char	flg_pwm8 = 0;
volatile unsigned char	flg_gpio = 0;
volatile unsigned char	flg_timer8 = 0;

unsigned char	mode = WAITING;				// IR condition
unsigned char	reader_on = 0;				// IR reader code on  length
unsigned char	reader_off = 0;				// IR reader code off length

void pwm8_1_isr(void);
void gpio_isr(void);
void timer8_1_isr(void);
void init(void);
void set_output(char *p);					// set LED PWM & RS232
void func_reader(void);						// function in IR reader code
void func_data(void);						// function in IR data code

void pwm8_1_isr(void)
{
	flg_pwm8 = 1;
}

void gpio_isr(void)
{
	flg_gpio = 1;
}

void timer8_1_isr(void)
{
	flg_timer8 = 1;
}

void init(void)
{
	/* LED PWM drive & timer(PWM8_1) */
	PWM8_1_EnableInt();										// timer interrupt
	PWM8_1_Start();											// R color
	PWM8_2_Start();											// G color
	PWM8_3_Start();											// B color
	
	/* send IR data by RS232 */
	TX8SW_1_Start();

	/* capture by timer8 */
	Timer8_1_Start();
	Timer8_1_EnableInt();
	
	M8C_EnableGInt;
	M8C_EnableIntMask(INT_MSK0, INT_MSK0_GPIO);
}

/*--------------------------------------
   set output LED PWM & RS232 of IR data
----------------------------------------*/
void set_output(char	*p)
{
	unsigned char	i;
	
	/* check custom code */
	
	/* check data code */
	if((*(p+2) + *(p+3)) != 0xff){
		return;
	}
	
	/* set data */
	PWM8_1_WritePulseWidth(color_chart[*(p+2)/10].r);
	PWM8_2_WritePulseWidth(color_chart[*(p+2)/10].g);
	PWM8_3_WritePulseWidth(color_chart[*(p+2)/10].b);

	/* send data on RS232 */
	for(i=0;i<4;i++){
		TX8SW_1_PutSHexByte(*(p+i));
		TX8SW_1_PutChar(' ');
	}
	TX8SW_1_PutCRLF();
}

/*--------------------------------------
   function in IR reader code
----------------------------------------*/
void func_reader(void)
{	
	if(flg_pwm8 == 1){										// timer 139us -> measure READER code length
		flg_pwm8 = 0;
		if((PRT0DR & 0x20) == 0){							// IR 'L'
			reader_on++;
		}
		if((PRT0DR & 0x20) != 0){							// IR 'H'
			if(reader_on > 50 && reader_on < 80){			// 7.0ms - 11.1ms(typ 9.0ms)
				reader_off++;
			}else{
				mode = WAITING;
				reader_on = 0;
				reader_off = 0;
			}
		}
	}
	if(flg_gpio == 1){
		flg_gpio = 0;
		if(reader_off > 16 && reader_off < 48){				// 2.2ms - 6.7ms(typ 4.5ms)
			mode = DATA;
			
			/* setup first capture mode */
			Timer8_1_Stop();
			Timer8_1_WritePeriod(255);
			Timer8_1_Start();
			flg_timer8 = 0;
		}else{
			mode = WAITING;
			reader_on = 0;
			reader_off = 0;
		}
	}
}

/*--------------------------------------
   function in IR data code
----------------------------------------*/
void func_data(void)
{
	static unsigned char	ir_bit = 0;						// IR stream index
	static unsigned char	ir_byte = 0;					// IR byte data
	       unsigned char	ir_bit_time = 0;				// IR bit time(length)
	static unsigned char	ir_buf[5];						// IR flame data(custom & data code)

	if(flg_pwm8 == 1){
		flg_pwm8 = 0;
	}
	if(flg_timer8 == 1){
		flg_timer8 = 0;
		ir_bit_time = Timer8_1_bReadCompareValue();			// result IR bit length
		
		/* setup next capture mode */
		Timer8_1_Stop();
		Timer8_1_WritePeriod(255);
		Timer8_1_Start();
				
		/* check IR bit length -> '1' or '0' */
		if(ir_bit_time > 43 && ir_bit_time < 129){			// typ 2.24ms -> '1'
			ir_byte = (ir_byte >> 1) | 0x80;
		}else if(ir_bit_time >= 129 && ir_bit_time < 215){	// typ 1.12ms -> '0'
			ir_byte = (ir_byte >> 1);
		}else{
			mode = WAITING;
			ir_bit = 0;
			ir_byte = 0;
		}
				
		/* finish 1byte */
		if((ir_bit & 0x07) == 7){
			ir_buf[ir_bit>>3] = ir_byte;
			ir_byte = 0;
		}
		ir_bit++;
				
		/* finish IR flame(include stop bit) */
		if(ir_bit == 32){
			ir_bit = 0;
			ir_byte = 0;
			set_output(ir_buf);								// set IR data -> LED PWM & RS232
			mode = WAITING;
		}
	}
}

void main(void)
{
	unsigned int	flame_time = 0;

	init();
	
	while(1){
		if(flg_pwm8 == 1){									// flg_pwm8 cleared by func_reader() or func_data()
			if(mode == READER || mode == DATA){				// measure IR flame length
				flame_time++;
			}
		}
		
		/* check flame length < 108ms */
		if(flame_time > 8308){
			flame_time = 0;
			mode = WAITING;
		}
		
		/* check condition */
		switch(mode){
		case WAITING:
			if(flg_gpio ==1){								// occure IR stream
				flg_gpio = 0;
				mode = READER;
			}
			
			/* initalize */
			reader_on  = 0;
			reader_off = 0;
			flame_time = 0;
			break;
			
		case READER:
			func_reader();
			break;
			
		case DATA:
			func_data();									// data ok -> set IR data to LED PWM & RS232
			break;
			
		default:
			mode = WAITING;
			break;
		}
	}
}
